import unittest
import os
from ybc_scene import *
from ybc_exception import *


class MyTestCase(unittest.TestCase):
    path_pic = os.path.join(os.path.split(os.path.abspath(__file__))[0], 'test.jpg')

    def test_scene_recog(self):
        self.assertEqual(scene_recog(MyTestCase.path_pic), [{'label_id': '室内', 'label_confd': '66%'}, {'label_id': '物品', 'label_confd': '33%'}])

    def test_object_recog(self):
        self.assertEqual(object_recog(MyTestCase.path_pic), [{'label_id': '杯子', 'label_confd': '54%'}, {'label_id': '茶杯', 'label_confd': '17%'}, {'label_id': '意式浓缩咖啡', 'label_confd': '12%'}, {'label_id': '室内', 'label_confd': '7%'}, {'label_id': '咖啡机', 'label_confd': '7%'}])

    def test_scene_recog_exValue(self):
        with self.assertRaisesRegex(ParameterValueError, "^参数数值错误 : 调用scene_recog方法时，'filename'参数不在允许范围内。$"):
            scene_recog('')

    def test_scene_recog_exType(self):
        with self.assertRaisesRegex(ParameterTypeError, "^参数类型错误 : 调用scene_recog方法时，'filename'参数类型错误。$"):
            scene_recog(1)

    def test_object_recog_exValue(self):
        with self.assertRaisesRegex(ParameterValueError, "^参数数值错误 : 调用object_recog方法时，'filename'参数不在允许范围内。$"):
            object_recog('')

    def test_object_recog_exType(self):
        with self.assertRaisesRegex(ParameterTypeError, "^参数类型错误 : 调用object_recog方法时，'filename'参数类型错误。$"):
            object_recog(1)


if __name__ == '__main__':
    unittest.main()



import requests
import os
import math
import sys
import ybc_config
from ybc_exception import *

__PREFIX = ybc_config.config['prefix']
__SCENE_URL = __PREFIX + ybc_config.uri + '/sceneRecognition'
__OBJECT_URL = __PREFIX + ybc_config.uri + '/objectRecognition'


def scene_recog(filename=''):
    """
    功能：场景识别。

    参数：filename: 待识别图片文件。

    返回：识别结果及置信度字典组成的列表。
    """
    if not isinstance(filename, str):
        raise ParameterTypeError(function_name=sys._getframe().f_code.co_name, error_msg="'filename'")
    if filename == '':
        raise ParameterValueError(function_name=sys._getframe().f_code.co_name, error_msg="'filename'")

    try:
        url = __SCENE_URL
        return _imageRecognition(filename, url)
    except (ParameterTypeError, ParameterValueError) as e:
        raise e
    except Exception as e:
        raise InternalError(e, 'ybc_scene')


def object_recog(filename=''):
    """
    功能：物体识别

    参数：filename: 待识别图片文件

    返回：识别结果及置信度字典组成的列表
    """
    if not isinstance(filename, str):
        raise ParameterTypeError(function_name=sys._getframe().f_code.co_name, error_msg="'filename'")
    if filename == '':
        raise ParameterValueError(function_name=sys._getframe().f_code.co_name, error_msg="'filename'")

    try:
        url = __OBJECT_URL
        return _imageRecognition(filename, url)
    except (ParameterValueError, ParameterTypeError) as e:
        raise e
    except Exception as e:
        raise InternalError(e, 'ybc_scene')


def _imageRecognition(filename='', url=''):
    if not filename:
        raise ParameterValueError(function_name=sys._getframe().f_code.co_name, error_msg="'filename'")

    ybc_config.resize_if_too_large(filename)
    filepath = os.path.abspath(filename)
    files = dict()
    with open(filepath, 'rb') as f:
        files['file'] = f.read()

    for i in range(3):
        r = requests.post(url, files=files)
        if r.status_code == 200:
            res = r.json()
            if res['errno'] == 0 and res['tags']:
                res_list = []
                sum = 0
                for val in res['tags']:
                    res_list.append({'label_id': val['value'],
                                     'label_confd': val['confidence']})
                    sum += val['confidence']
                for val in res_list:
                    val['label_confd'] = str(math.floor(val['label_confd']/sum * 100)) + '%'

                return res_list
            else:
                return "图片中未识别到场景或物体"

    raise ConnectionError('识别物体或场景信息失败', r._content)


def main():
    res = object_recog('test.jpg')
    print(res)
    res = scene_recog('test.jpg')
    print(res)


if __name__ == '__main__':
    main()
